/*

   fl0p - fingerprint / flow structures
   ------------------------------------
   
   Copyright (C) 2006 by Michal Zalewski <lcamtuf@coredump.cx>
   
 */

#ifndef _HAVE_FPENTRY_H
#define _HAVE_FPENTRY_H

#define ITEM_EOF        0	/* End of signature */
#define ITEM_CLI	1	/* Client packet (C) */
#define ITEM_SERV	2	/* Server packet (S) */

#define ITEM_VAL_ANY	0	/* * */
#define ITEM_VAL_NORM	1	/* nn */
#define ITEM_VAL_REF	2	/* #nn*mm */

#define ITEM_D_NONE	0	/* First element */
#define ITEM_D_SHORT	1 	/* Short delay */
#define ITEM_D_LONG	2 	/* Long delay */

/* Signature element: */
struct sig_item {
  _u8  type:4,		/* Element type (ITEM_*) */
       valtype:4;	/* Parameter type (ITEM_VAL_*) */
  _u16 value;		/* Parameter value */
  _u16 range;		/* Match range (+/- nn) */
  _u8  d_prev;		/* Delay from previous (ITEM_D_*) */
};


#define SIG_FLAG_NONE	0
#define SIG_FLAG_SET	1
#define SIG_FLAG_ZERO	2


/* Single sgnature entry: */
struct signature {
  _u8* name;		/* Traffic description (NULL for silent) */
  _u8  proto;		/* Protocol ID (IPPROTO_*) */
  _u16 port;		/* Destination port (0 - any) */
  _u8  siglen;		/* Signature element count */
  _u8  f_check_mode:4,	/* Flag checking mode (SIG_FLAG_*) */
       f_set_mode:4; 	/* Flag setting mode (SIG_FLAG_*) */
  _u32 f_check,		/* Which flags to check */
       f_set;		/* Which flags to set */
  struct sig_item 
    el[MAXSIGLEN+1];	/* Signature entry */
  _u8 need_open,	/* Session init required (OPEN) */
      need_close,	/* Session termination required (CLOSE) */
      no_fuzzy;		/* Fuzzy matching off (EXACT) */
};


/* Packet flow history entry: */
struct stream_item {
  _u8  type; 		/* Packet type (ITEM_*) */
  _u16 value;		/* Packet size */
  _u8  d_prev;		/* Delay from prev (ITEM_D_*) */
};


/* Flow stream entry: */
struct fl0p_stream {
  _u8  proto;		/* Protocol ID (IPPROTO_*) */
  _u32 caddr, saddr;	/* Client / server addresses */
  _u16 cport, sport;	/* Client / server ports (0 for ICMP) */
  _u32 pcount;		/* Processed packet count */
  _u64 dcount;		/* Processed payload byte count */
  _u32 start_time,	/* Session start time (unix) */
       last_time,	/* Last packet time (unix) */
       last_usec;	/* + microseconds */

  _u32 flag;		/* User-set flag value */

  struct stream_item 
    head[MAXSIGLEN],	/* Flow data for first MAXISGLEN packets */
    tail[MAXSIGLEN*2];	/* Working "tail" for signature matching */
    
  _u32 matched;		/* Matched signature counter */

};


#define BUCK(saddr,daddr,sport,dport) \
	( ((saddr) ^ (daddr) ^ (sport) ^ (dport)) \
                  % (FLOW_BUCKETS + 1) )


#endif /* !_HAVE_FPENTRY_H */

